package group_r

import (
	"fmt"
	"git.hilo.cn/hilo-common/domain"
	"git.hilo.cn/hilo-common/mycontext"
	"git.hilo.cn/hilo-common/resource/mysql"
	"git.hilo.cn/hilo-common/utils"
	"github.com/gin-gonic/gin"
	"github.com/spf13/cast"
	"gorm.io/gorm"
	"hilo-group/_const/enum/game_e"
	"hilo-group/_const/enum/group_e"
	"hilo-group/cv/group_cv"
	"hilo-group/cv/group_power_cv"
	"hilo-group/cv/medal_cv"
	"hilo-group/cv/user_cv"
	"hilo-group/domain/cache/res_c"
	"hilo-group/domain/cache/room_c"
	"hilo-group/domain/model/game_m"
	"hilo-group/domain/model/group_m"
	"hilo-group/domain/model/mgr_m"
	"hilo-group/domain/model/res_m"
	"hilo-group/domain/model/rocket_m"
	"hilo-group/domain/model/user_m"
	"hilo-group/domain/model/version_m"
	"hilo-group/domain/service/group_s"
	"hilo-group/myerr"
	"hilo-group/myerr/bizerr"
	"hilo-group/req"
	"hilo-group/resp"
	"sort"
	"strings"
)

// @Tags 群组
// @Summary 查询我创建的群组
// @Accept application/x-www-form-urlencoded
// @Param token header string true "token"
// @Param nonce header string true "随机数字"
// @Success 200 {object} group_cv.JoinedGroupInfo
// @Router /v1/imGroup/ownGroup [get]
func GetOwnGroup(c *gin.Context) (*mycontext.MyContext, error) {
	myContext := mycontext.CreateMyContext(c.Keys)

	userId, err := req.GetUserId(c)
	if err != nil {
		return myContext, err
	}

	model := domain.CreateModelContext(myContext)

	groupIds, err := group_m.GetGroups(model.Db, userId, group_e.GROUP_OWNER)
	if err != nil {
		return myContext, err
	}
	if len(groupIds) <= 0 {
		resp.ResponseOk(c, []group_cv.JoinedGroupInfo{})
		return myContext, nil
	}

	myService := domain.CreateService(myContext)
	result, _, err := group_cv.BuildJoinedGroupInfo(myService, userId, groupIds, 1, 1)
	if err != nil {
		return myContext, err
	}

	resp.ResponseOk(c, result)
	return myContext, nil
}

// @Tags 群组
// @Summary 查询某人的（公开）群组（新）
// @Accept application/x-www-form-urlencoded
// @Param token header string true "token"
// @Param nonce header string true "随机数字"
// @Param userExternalId path string true "用户ExternalId"
// @Success 200 {object} group_cv.TheirGroupsInfo
// @Router /v1/imGroup/theirGroup/{userExternalId} [get]
func GetTheirGroups(c *gin.Context) (*mycontext.MyContext, error) {
	myContext := mycontext.CreateMyContext(c.Keys)

	externalId := c.Param("userExternalId")
	if len(externalId) <= 0 {
		return myContext, myerr.NewSysError("userExternalId 为必填项")
	}

	model := domain.CreateModelContext(myContext)

	user, err := user_m.GetUserByExtId(model, externalId)
	if err != nil {
		return myContext, err
	}
	userId := user.ID
	if userId == 0 {
		return myContext, bizerr.InvalidParameter
	}

	data, err := group_m.GetJoinedGroups(model.Db, userId)
	if err != nil {
		return myContext, err
	}

	result := group_cv.TheirGroupsInfo{Total: 0}

	groupIds := make([]string, 0)
	for _, i := range data {
		groupIds = append(groupIds, i.GroupId)
	}
	groupInfo, err := group_m.BatchGetGroupInfo(model, groupIds)
	if err != nil {
		return myContext, err
	}
	owners := make([]uint64, 0)
	for _, i := range groupInfo {
		owners = append(owners, i.Owner)
	}

	theirRoles, err := group_m.GetRolesByUser(model, userId)
	if err != nil {
		return myContext, err
	}
	model.Log.Info("GetTheirGroups: theirRoles - ", theirRoles)

	powerIds, powerNames, err := group_power_cv.BatchGetGroupPower(model.Db, owners)
	if err != nil {
		return myContext, err
	}
	groupMedals, err := group_m.BatchGetMedals(model.Db, groupIds)
	if err != nil {
		return myContext, err
	}
	resMedal, err := res_m.MedalGetAllMap(model.Db)
	if err != nil {
		return myContext, err
	}

	roomMicUserMap, err := group_m.BatchGetAllMicUser(model, groupIds)
	if err != nil {
		return myContext, err
	}
	model.Log.Infof("GetTheirGroups: groupIds %v, roomMicUserMap %v", groupIds, roomMicUserMap)

	uids := owners
	micUsersMap := make(map[string][]uint64, 0)
	for _, i := range groupIds {
		micUsersMap[i] = make([]uint64, 0)

		u := roomMicUserMap[i]
		if len(u) >= 4 {
			micUsersMap[i] = u[0:4]
		} else {
			micUsersMap[i] = u
		}
		uids = append(uids, micUsersMap[i]...)
	}

	uids = utils.UniqueSliceUInt64(uids)
	userTiny, err := user_cv.GetUserTinyMap(uids)
	if err != nil {
		return myContext, err
	}

	model.Log.Infof("GetTheirGroups: userTinyMap: %v", userTiny)

	roomCount, err := group_m.BatchGetRoomCount(model, groupIds)
	if err != nil {
		return nil, err
	}

	onMic, err := group_cv.CheckMicHasUserByGroup(groupIds)
	if err != nil {
		return myContext, err
	}
	model.Log.Info("GetTheirGroups: onMic - ", onMic)

	countryInfo, err := res_c.GetCountryIconMap(model)
	if err != nil {
		return myContext, err
	}

	supportLevels, err := group_s.NewGroupService(myContext).GetWeekMaxSupportLevelMap()
	if err != nil {
		return myContext, err
	}

	rr := rocket_m.RocketResult{}
	maxStageMap, err := rr.GetMaxStage(mysql.Db, groupIds)
	if err != nil {
		return myContext, err
	}

	visitCount, err := group_m.BatchGetRoomVisitCount(model.Log, groupIds)
	if err != nil {
		return myContext, err
	}

	roomEnterTime, err := room_c.GetUserRoomVisit(userId)
	if err != nil {
		return myContext, err
	}
	model.Log.Infof("BuildJoinedGroupInfo, roomEnterTime: %v", roomEnterTime)
	// 正在进行的游戏
	games := game_m.GetNotEndGamesMap(model)

	for _, i := range data {
		gi := groupInfo[i.GroupId]
		// 只要公开的群
		if len(gi.Password) == 0 {
			result.Total++

			role := group_e.GROUP_MEMBER
			r, exists := theirRoles[i.GroupId]
			if exists {
				role = r
			}

			micUsers := make([]user_cv.CvUserTiny, 0)
			for _, j := range micUsersMap[i.GroupId] {
				micUsers = append(micUsers, userTiny[j])
			}

			var maxStage *uint16 = nil
			if s, ok := maxStageMap[i.GroupId]; ok {
				maxStage = &s
			}

			medals := make([]medal_cv.PicElement, 0)
			if m, ok := groupMedals[i.GroupId]; ok {
				for _, j := range m {
					mId := uint32(j)
					if e, ok := resMedal[mId]; ok {
						medals = append(medals, medal_cv.PicElement{
							PicUrl: e.PicUrl,
						})
					}
				}
			}
			// 补上房间流水勋章
			var pe *medal_cv.PicElement
			_, pe, err = medal_cv.GetGroupConsumeMedal(model, i.GroupId)
			if err != nil {
				model.Log.Infof("GetTheirGroups: GetGroupConsumeMedal: %s", err.Error())
			} else if pe != nil {
				medals = append(medals, medal_cv.PicElement{PicUrl: pe.PicUrl})
			}

			elem := group_cv.JoinedGroupInfo{
				PopularGroupInfo: group_cv.PopularGroupInfo{
					GroupInfo: group_cv.GroupInfo{
						GroupBasicInfo: group_cv.GroupBasicInfo{
							GroupId:             gi.TxGroupId,
							Name:                gi.Name,
							Introduction:        gi.Introduction,
							Notification:        gi.Notification,
							FaceUrl:             gi.FaceUrl,
							Code:                gi.Code,
							CountryIcon:         countryInfo[gi.Country],
							SupportLevel:        supportLevels[i.GroupId],
							GroupInUserDuration: visitCount[i.GroupId],
							MicNumType:          int(gi.MicNumType),
							GroupMedals:         medals,
						},
						//HasOnMic:       onMicScore[i.GroupId] != 0,
						GroupPowerId:   powerIds[gi.Owner],
						GroupPowerName: powerNames[gi.Owner],
					},
					MicUsers:      micUsers,
					RoomUserCount: uint(roomCount[i.GroupId]),
					MaxStage:      maxStage,
					GameTypes:     games[gi.TxGroupId],
				},
				LastEnterTime: roomEnterTime[i.GroupId],
			}

			if role == group_e.GROUP_OWNER {
				result.OwnGroups = append(result.OwnGroups, elem)
			} else {
				result.Groups = append(result.Groups, elem)
			}
		}
	}

	// fixme: 按最近进入时间？
	sort.Slice(result.OwnGroups, func(i, j int) bool {
		return result.OwnGroups[i].LastEnterTime > result.OwnGroups[j].LastEnterTime ||
			result.OwnGroups[i].LastEnterTime == result.OwnGroups[j].LastEnterTime && result.OwnGroups[i].GroupId < result.OwnGroups[j].GroupId
	})
	// fixme:按角色排？
	sort.Slice(result.Groups, func(i, j int) bool {
		return theirRoles[result.Groups[i].GroupId] > theirRoles[result.Groups[j].GroupId] ||
			theirRoles[result.Groups[i].GroupId] == theirRoles[result.Groups[j].GroupId] && result.Groups[i].GroupId < result.Groups[j].GroupId
	})

	resp.ResponseOk(c, result)
	return myContext, nil
}

// @Tags 群组
// @Summary 客户端组件加载完成
// @Accept application/x-www-form-urlencoded
// @Param token header string true "token"
// @Param nonce header string true "随机数字"
// @Param groupId path string true "群ID"
// @Success 200
// @Router /v1/imGroup/pluginReady/{groupId} [put]
func PluginReady(c *gin.Context) (*mycontext.MyContext, error) {
	myContext := mycontext.CreateMyContext(c.Keys)

	userId, externalId, err := req.GetUserIdAndExtId(c, myContext)
	if err != nil {
		return myContext, err
	}

	groupId := c.Param("groupId")
	if len(groupId) <= 0 {
		return myContext, myerr.NewSysError("groupId 为必填项")
	}

	model := domain.CreateModelContext(myContext)

	groupId, err = group_m.ToImGroupId(model, groupId)
	if err != nil {
		return myContext, err
	}

	model.Log.Infof("PluginReady user %d, %s", userId, externalId)

	/** fixme：这协议没用了？

	go func(gi *group_m.GroupInfo) {
		defer func() {
			if r := recover(); r != nil {
				//打印错误堆栈信息
				mylogrus.MyLog.Errorf("PluginReady SYSTEM ACTION PANIC: %v, stack: %v", r, string(debug.Stack()))
			}
		}()

		if err := myService.OnJoinGroup(externalId, groupId); err != nil {
			model.Log.Infof("PluginReady, OnJoinGroup err %v", err)
		}
	}(groupInfo)*/

	resp.ResponseOk(c, nil)
	return myContext, nil
}

// @Tags 群组
// @Summary 查询房间信息
// @Accept application/x-www-form-urlencoded
// @Param token header string true "token"
// @Param nonce header string true "随机数字"
// @Param groupId path string true "群ID"
// @Success 200 {object} cv.RoomInfo
// @Router /v1/imGroup/roomInfo/{groupId} [get]
func GetRoomInfo(c *gin.Context) (*mycontext.MyContext, error) {
	myContext := mycontext.CreateMyContext(c.Keys)

	groupId := c.Param("groupId")
	if len(groupId) <= 0 {
		return myContext, myerr.NewSysError("groupId 为必填项")
	}

	model := domain.CreateModelContext(myContext)

	groupInfo, err := group_m.GetInfoByTxGroupId(model, groupId)
	if err != nil {
		return myContext, err
	}
	if groupInfo == nil {
		return myContext, bizerr.GroupNotFound
	}
	groupId = groupInfo.ImGroupId

	memberCount, err := group_m.GetMemberCount(model.Db, groupId)
	if err != nil {
		return myContext, err
	}

	countryInfo, err := res_c.GetCountryIconMap(model)
	if err != nil {
		return myContext, err
	}

	// 版本控制
	_deviceType, _ := c.Get(mycontext.DEVICETYPE)
	_appVersion, _ := c.Get(mycontext.APP_VERSION)
	deviceType, appVersion := cast.ToString(_deviceType), cast.ToString(_appVersion)
	vc := version_m.GetVersionControlByPlatform(model, deviceType)
	allow, err := utils.CompareVersion(appVersion, fmt.Sprintf("<= %s", vc.AuditVersion))
	model.Log.Infof("GetRoomInfo CompareVersion appVersion:%v,auditVersion:%v,allow:%v,err:%v", appVersion, vc.AuditVersion, allow, err)

	themeUrl := ""
	var themeId uint64 = 0
	var themeType uint8 = 1
	if groupInfo.ThemeId > 0 {
		//官方主题
		// 找不到的，默认为空（可能被后台删了）
		themeType = 1
		themes, _ := res_m.GroupThemeGetAll(model.Db)
		for _, i := range themes {
			if i.ID == uint64(groupInfo.ThemeId) {
				themeUrl = i.Url
				themeId = i.ID
				break
			}
		}
	} else {
		//可能是自定义主题
		themeId, themeUrl, err = group_m.GetShowCustomTheme(model, groupInfo.ImGroupId)
		if err != nil {
			return myContext, err
		}
		if themeId != 0 {
			themeType = 2
		}
	}

	result := group_cv.RoomInfo{
		GroupBasicInfo: group_cv.GroupBasicInfo{
			GroupId:        groupInfo.TxGroupId,
			Name:           groupInfo.Name,
			Introduction:   groupInfo.Introduction,
			Notification:   groupInfo.Notification,
			FaceUrl:        groupInfo.FaceUrl,
			MemberNum:      memberCount,
			Code:           groupInfo.Code,
			CountryIcon:    countryInfo[groupInfo.Country],
			MicNumType:     int(groupInfo.MicNumType),
			TouristMic:     groupInfo.TouristMic,
			TouristSendMsg: groupInfo.TouristSendMsg,
			TouristSendPic: groupInfo.TouristSendPic,
			MemberFee:      groupInfo.MemberFee,
		},
		MicOn:       groupInfo.MicOn,
		LoadHistory: groupInfo.LoadHistory,
		ThemeId:     themeId,
		ThemeUrl:    themeUrl,
		ThemeType:   themeType,
		Role:        group_e.GROUP_VISITOR,
		GameConfig:  &game_m.GameConfig{Ludo: &game_m.GameConfigDiamond{Diamond: game_e.GameLudoDiamondList}, Uno: &game_m.GameConfigDiamond{Diamond: game_e.GameLudoDiamondList}},
	}

	roles, _, err := group_m.GetRolesInGroup(model, groupInfo.ImGroupId)
	if err != nil {
		return myContext, err
	}

	userId, err := req.GetUserId(c)
	if err != nil {
		return myContext, err
	}

	isGroupMember, err := group_m.IsGroupMember(model.Db, groupId, userId)
	if err != nil {
		return myContext, err
	}
	if isGroupMember {
		result.Role = group_e.GROUP_MEMBER
	}

	emptyStr := ""
	if len(groupInfo.Password) > 0 {
		// 代表有密码
		result.Password = &emptyStr
	}

	//userIds := make([]uint64, 0)
	for u, r := range roles {
		//userIds = append(userIds, u)

		if u == userId {
			if r == group_e.GROUP_OWNER || r == group_e.GROUP_MANAGER {
				// 如果用户是OW或经理，可以看到密码，同时设置角色
				result.Role = r

				if len(groupInfo.Password) > 0 {
					result.Password = &groupInfo.Password
				}
			} else if r == group_e.GROUP_ADMIN {
				// 如果用户是管理员，仅设置角色
				result.Role = r
			}
		}
	}

	roles, orderList, err := group_m.GetRolesInGroup(model, groupId)
	if err != nil {
		return myContext, err
	}

	users, err := user_cv.GetUserTinyMap(orderList)
	if err != nil {
		return myContext, err
	}

	for _, i := range orderList {
		result.RoleMembers = append(result.RoleMembers, group_cv.SimpleRoleInfo{
			ExternalId: users[i].ExternalId,
			Role:       roles[i],
		})
	}

	// ---- 以下部分相对没那么重要，失败也不返回 ----

	supportLevels, err := group_s.NewGroupService(myContext).GetWeekMaxSupportLevelMap()
	if err != nil {
		model.Log.Infof("GetRoomInfo: GetWeekMaxSupportLevelMap: %s", err.Error())
	}
	result.SupportLevel = supportLevels[groupId]

	result.DiceNum = group_e.GROUP_DICE_NUM_DEFAULT
	result.DiceType = 1 // 1: [0-9] 2:[1-6]
	gs := group_m.GroupSetting{GroupId: groupId}
	err = gs.Get(model.Db)
	if err == nil {
		result.DiceNum = gs.DiceNum
		result.DiceType = gs.DiceType
	} else if err != gorm.ErrRecordNotFound {
		model.Log.Infof("GetRoomInfo: GroupSetting: %s", err.Error())
	}

	result.WelcomeText, _, _, err = group_s.NewGroupService(myContext).GetWelcomeText(groupInfo)
	if err != nil {
		return myContext, err
	}
	result.WelcomeText = strings.ReplaceAll(result.WelcomeText, "@%s", "")

	// 补上房间流水勋章
	var pe *medal_cv.PicElement
	result.TotalConsume, pe, err = medal_cv.GetGroupConsumeMedal(model, groupId)
	if err != nil {
		model.Log.Infof("GetRoomInfo: GetGroupConsumeMedal: %s", err.Error())
	} else if pe != nil {
		result.GroupMedals = append(result.GroupMedals, medal_cv.PicElement{PicUrl: pe.PicUrl})
	}

	resMedals := []res_m.ResMedal{}
	err = mysql.Db.Model(&res_m.ResMedal{}).
		Joins("INNER JOIN group_medal m ON m.res_medal_id = res_medal.id").
		Where("m.im_group_id = ?", groupId).
		Order("m.id desc").
		Find(&resMedals).Error
	if err == nil {
		for _, r := range resMedals {
			result.GroupMedals = append(result.GroupMedals, medal_cv.PicElement{
				PicUrl:  r.PicUrl,
				SvgaUrl: r.SvgaUrl,
			})
		}
	} else {
		model.Log.Infof("GetRoomInfo: group_medal: %s", err.Error())
	}
	// 获取国家信息
	country, area, err := user_m.GetUserCountryArea(model, userId)
	if err != nil {
		model.Log.Errorf("GetRoomInfo 获取国家资源错误 userId:%d, err:%v", userId, err)
		return myContext, err
	}

	rb := mgr_m.RoomBanner{Status: mysql.YES}
	rows, err := rb.GetRoomBanners(model.Db, allow)
	if err == nil {
		for _, i := range rows {
			// 是否在banner设定区域内
			if (i.Area == 1 || i.Area == 2) && area != i.Area { // 不在区域内
				continue
			}
			if i.Area == 3 && !utils.IsInStringList(country, i.CountryList) {
				continue
			}

			result.Banners = append(result.Banners, group_cv.BannerElement{
				H5Url:     i.Url,
				BannerUrl: i.Image,
			})
		}
	} else {
		model.Log.Infof("GetRoomInfo: GetRoomBanners: %s", err.Error())
	}
	result.LuckyWheel, err = group_cv.GetLuckWheelState(model, groupId)
	if err != nil {
		model.Log.Infof("GetRoomInfo: GetLuckWheelState: %s", err.Error())
	}
	// 群主的详情
	result.Owner, err = user_cv.GetUserDetail(model, groupInfo.Owner, userId)
	if err != nil {
		model.Log.Errorf("GetRoomInfo: GetUserBase: %s", err.Error())
	}

	resp.ResponseOk(c, result)
	return myContext, nil
}

// @Tags 群组
// @Summary 查询群管理角色（经理、管理）
// @Accept application/x-www-form-urlencoded
// @Param token header string true "token"
// @Param nonce header string true "随机数字"
// @Param groupId path string true "群ID"
// @Success 200 {object} group_cv.MemberDetail
// @Router /v1/imGroup/admin/{groupId} [get]
func GetGroupAdmin(c *gin.Context) (*mycontext.MyContext, error) {
	myContext := mycontext.CreateMyContext(c.Keys)

	groupId := c.Param("groupId")
	if len(groupId) <= 0 {
		return myContext, myerr.NewSysError("groupId 为必填项")
	}

	myUserId, err := req.GetUserId(c)
	if err != nil {
		return myContext, err
	}

	model := domain.CreateModelContext(myContext)

	groupId, err = group_m.ToImGroupId(model, groupId)
	if err != nil {
		return myContext, err
	}

	roles, orderList, err := group_m.GetRolesInGroup(model, groupId)
	if err != nil {
		return myContext, err
	}

	userIds := make([]uint64, 0)
	for _, i := range orderList {
		if roles[i] == group_e.GROUP_MANAGER || roles[i] == group_e.GROUP_ADMIN {
			userIds = append(userIds, i)
		}
	}

	result := make([]group_cv.MemberDetail, 0)
	if len(userIds) > 0 {
		users, err := user_cv.BatchGetUserExtend(model, userIds, myUserId)
		if err != nil {
			return myContext, err
		}

		for _, i := range userIds {
			result = append(result, group_cv.MemberDetail{
				CvUserExtend: users[i],
				Role:         roles[i],
			})
		}
	}

	resp.ResponseOk(c, result)
	return myContext, nil
}
